#!/usr/bin/env python
# -*- coding: UTF-8 -*-


# part of battery-status project
# battery-status - Battery Status for the GNOME desktop
# Copyright (c) 2010 Ivan Zorin <ivan.a.zorin@gmail.com>
# see README for more information


import appindicator
import pygtk
import gtk
import gconf
import platform
import dbus
import os
import time
import sys
import gobject


class BatteryIndicator(appindicator.Indicator):
	"""GNOME Battery Status indicator - shows battery status in GNOME indicator area"""
	
	
	# bool var. for default status of apport package
	package_apport = False
	# bool var. for default status of gnome-screensaver package
	package_screensaver = True
	# bool var. for default status of gnome-session-bin package
	package_session = True
	# bool var. for default status of gnome-power-manager package
	package_power = True
	# bool var. for default status of gnome-applets (with cpufreq-selector as backend for switching CPU frequency) package
	package_cpufreq = True
	# bool var. for detect D-Bus UPower/DeviceKit-Power error
	power_error = False
	# bool var. for detect CPU frequency scaling error
	cpufreq_error = False
	# bool var. for battery available status
	no_battery = False
	# bool var. for set up correct background when adding applet on panel
	option_panel_bgstate = 0
	# bool var. for low battery state trigger
	on_low_battery = False
	# bool var. for lock main menu item when battery information dialog opens
	lock_battery_status = False
	# bool var. of suspend support for sleep actions
	can_suspend = True
	# bool var. of hibernate support for sleep actions
	can_hibernate = True
	# power source device
	power_dev = ""
	# current power mode (in indicator)
	powermode = ""
	# default set of CPU items for scaling
	cpu_items = ""
	
	
	def __init__(self, indicator):
		"""Create indicator"""
		self.indicator = indicator
		self.init_widgets()
		self.init_packages()
		self.init_gconf_settings()
		self.init_first_run()
		self.init_cpufreq()
		self.init_main_menu()
		self.init_power_dbus()
	
	
	def init_widgets(self):
		"""Create widgets for indicator"""
		# init battery indicator state
		self.icon_name = "gpm-ac-adapter"
		self.indicator.set_status(appindicator.STATUS_ACTIVE)
		self.indicator.set_icon(self.icon_name)
		# get current icon theme for icons
		self.icon_theme = gtk.icon_theme_get_default()
		self.icon_theme.append_search_path('/usr/share/gnome-power-manager/icons')
		self.icon_theme.connect("changed", self.power_dbus_signal)
		# icon of gnome-power-manager
		self.pixbuf_gpm_32 = self.icon_theme.load_icon("gnome-power-manager", 32, gtk.ICON_LOOKUP_FORCE_SVG)
		### widgets for battery status dialog
		# label for battery capacity information
		self.label_capacity_data = gtk.Label()
		self.label_capacity_data.set_justify(gtk.JUSTIFY_CENTER)
		# label for battery charge information
		self.label_charge_data = gtk.Label()
		self.label_charge_data.set_justify(gtk.JUSTIFY_RIGHT)
		# label for battery technical information
		self.label_info_data = gtk.Label()
		self.label_info_data.set_justify(gtk.JUSTIFY_CENTER)
		# label for battery information text
		self.label_info = gtk.Label()
		self.label_info.set_justify(gtk.JUSTIFY_LEFT)
		# progress bar of battery capacity
		self.progress_capacity = gtk.ProgressBar()
		self.progress_capacity.set_size_request(200, 24)
		# progress bar of battery charge
		self.progress_charge = gtk.ProgressBar()
		self.progress_charge.set_size_request(200, 24)
		# Powersave item for Power Mode
		self.item_mode_powersave = gtk.RadioMenuItem(None, "Powe_rsave")
		# Ondemand item for Power Mode
		self.item_mode_ondemand = gtk.RadioMenuItem(self.item_mode_powersave , "On_demand")
		# Conservative item for Power Mode
		self.item_mode_conservative = gtk.RadioMenuItem(self.item_mode_powersave , "N_ormal")
		# Performance item for Power Mode
		self.item_mode_performance = gtk.RadioMenuItem(self.item_mode_powersave , "P_erformance")
		# Power Modes items separator
		self.power_separator = gtk.SeparatorMenuItem()
	
	
	def init_packages(self):
		"""Detect packages"""
		dist = ""
		# detect current distribution
		try:
			# to get distroname in a python 2.6 way
			dist = platform.linux_distribution()[0]
		except:
			# get distroname in a deprecated way
			dist = platform.dist()[0]
		if dist == "Ubuntu" or dist == "debian":
			try:
				import apt
				cache = apt.Cache()
				package = cache["gnome-screensaver"]
				self.package_screensaver = package.isInstalled
				package = cache["gnome-session-bin"]
				self.package_session = package.isInstalled
				package = cache["gnome-power-manager"]
				self.package_power = package.isInstalled
				package = cache["gnome-applets"]
				self.package_cpufreq = package.isInstalled
				package = cache["apport-gtk"]
				self.package_apport = package.isInstalled
			except:
				pass
		elif dist == "Fedora":
			try:
				import rpm
				transaction = rpm.TransactionSet()
				matches = transaction.dbMatch()
				matches.pattern('name', rpm.RPMMIRE_GLOB, "gnome-screensaver")
				self.package_screensaver = False
				for match in matches:
					self.package_screensaver = True
				matches = transaction.dbMatch()
				matches.pattern('name', rpm.RPMMIRE_GLOB, "gnome-session")
				self.package_session = False
				for match in matches:
					self.package_session = True
				matches = transaction.dbMatch()
				matches.pattern('name', rpm.RPMMIRE_GLOB, "gnome-power-manager-extra")
				self.package_power = False
				for match in matches:
					self.package_power = True
				matches = transaction.dbMatch()
				matches.pattern('name', rpm.RPMMIRE_GLOB, "gnome-applets")
				self.package_cpufreq = False
				for match in matches:
					self.package_cpufreq = True
			except:
				pass
		else:
			pass
	
	
	def init_first_run(self):
		"""After-init hook"""
		# if user run indicator at first time and use gnome-power-manager icon in indicator area
		if self.option_firstrun and self.option_gpm_icon_policy != "never":
			# then show advice to remove icon from indicator area
			message_icon_tray = gtk.MessageDialog(parent = None, type = gtk.MESSAGE_QUESTION, buttons = gtk.BUTTONS_NONE, message_format = None)
			title_text = "Battery Status indicator has been added in Indicator area."
			message_text = 'Battery Status indicator shows state of battery in your laptop, but you also use Power Manager\'s battery icon for the same purpose - would you like to remove it?\n\nYou can always change Power Manager\'s battery icon settings and its behavior in "Power Management" Preferences'
			message_icon_tray.set_markup('<span weight="bold" size="medium">' + title_text + '</span>' + '\n\n' + message_text)
			message_icon_tray.set_icon(self.pixbuf_gpm_32)
			message_icon_tray.set_property("window-position", "center")
			message_icon_tray.set_keep_above(True)
			message_icon_tray.add_button("_Remove", 1)
			message_icon_tray.add_button("Ign_ore", 0)
			message_icon_tray.set_default_response(0)
			if message_icon_tray.run() == 1:
				self.g_conf.set_string("/apps/gnome-power-manager/ui/icon_policy", "never")
			message_icon_tray.destroy()
		self.g_conf.set_bool("/apps/battery_status/first_run", False)
	
	
	def init_cpufreq(self):
		"""Init detecting CPU frequency mode"""
		if self.package_cpufreq and self.option_cpufreq:
			try:
				### to get CPU frequency scaling information
				# get current governor
				powermode_file = open('/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor', 'r')
				self.powermode = powermode_file.read().rstrip('\n')
				powermode_file.close()
				# get list of CPUs
				cpu_data = os.popen("ls /sys/devices/system/cpu | grep '^cpu[0-9]*$' | sed 's,^cpu,,'")
				self.cpu_items = cpu_data.readlines()
				cpu_data.close()
				self.cpufreq_error = False
			except:
				self.cpufreq_error = True
		else:
			pass
	
	
	def init_gconf_settings(self):
		"""Getting indicator settings from GConf"""
		# set up connection to GConf
		self.g_conf = gconf.client_get_default()
		# check settings
		settings = self.g_conf.dir_exists("/apps/battery_status")
		# if GConf settings not available from schema file, set up defaults
		if not settings:
			# Show > submenu option
			self.g_conf.set_string("/apps/battery_status/show", "icon")
			# Text Size > submenu option
			self.g_conf.set_string("/apps/battery_status/text", "small")
			# option for icon behavior
			self.g_conf.set_string("/apps/battery_status/icon_policy", "always")
			# option for lock screen on suspend/hibernate
			self.g_conf.set_bool("/apps/battery_status/lock_screen", True)
			# option for show additional info in battery status dialog window
			self.g_conf.set_bool("/apps/battery_status/show_info", False)
			# option for show tooltip
			self.g_conf.set_bool("/apps/battery_status/show_tooltip", True)
			# option for show status of battery when charge time/lifetime not available
			self.g_conf.set_bool("/apps/battery_status/show_status", True)
			# option for show sleep actions (Suspend/Hibernate) in main menu
			self.g_conf.set_bool("/apps/battery_status/show_sleep", False)
			# option for show session actions (Logout/Shutdown) in main menu
			self.g_conf.set_bool("/apps/battery_status/show_session", False)
			# option for show gnome-power-manager related items in main menu
			self.g_conf.set_bool("/apps/battery_status/show_power", True)
			# option for low charge, when shows critical battery message window
			self.g_conf.set_int("/apps/battery_status/percentage_low", 8)
			# option for time of showing critical battery message window
			self.g_conf.set_int("/apps/battery_status/timer", 60)
			# first run - for show advice to remove gnome-power-managment tray icon, if in use.
			self.g_conf.set_bool("/apps/battery_status/first_run", True)
			# option for show power modes
			self.g_conf.set_bool("/apps/battery_status/show_powermode", True)
			# option for show Text Size settings
			self.g_conf.set_bool("/apps/battery_status/show_textsize", False)
			# option for show Show > and Text Size > settings
			self.g_conf.set_bool("/apps/battery_status/show_settings", True)
			# option for show power device in "Power Statistics" dialog according to current power source
			self.g_conf.set_bool("/apps/battery_status/power_source_sensitive", False)
			# option for reduce label with "hour" digit
			self.g_conf.set_bool("/apps/battery_status/reduce_hours", False)
			# option for Power Mode menu
			self.g_conf.set_bool("/apps/battery_status/powermode_as_submenu", False)
			# option for allow power management
			self.g_conf.set_bool("/apps/battery_status/power_manager", False)
		# get settings for indicator
		self.option_icon = self.g_conf.get_string("/apps/battery_status/icon_policy")
		self.option_lock = self.g_conf.get_bool("/apps/battery_status/lock_screen")
		self.option_info = self.g_conf.get_bool("/apps/battery_status/show_info")
		self.option_sleep = self.g_conf.get_bool("/apps/battery_status/show_sleep")
		self.option_session = self.g_conf.get_bool("/apps/battery_status/show_session")
		self.option_power = self.g_conf.get_bool("/apps/battery_status/show_power")
		self.option_percentage_low = self.g_conf.get_int("/apps/battery_status/percentage_low")
		self.option_timer = self.g_conf.get_int("/apps/battery_status/timer")
		self.option_firstrun = self.g_conf.get_bool("/apps/battery_status/first_run")
		self.option_cpufreq = self.g_conf.get_bool("/apps/battery_status/show_powermode")
		self.option_sensitive = self.g_conf.get_bool("/apps/battery_status/power_source_sensitive")
		self.option_gpm = self.g_conf.get_bool("/apps/battery_status/power_manager")
		if not self.option_icon:
			self.g_conf.set_string("/apps/battery_status/icon_policy", "always")
			self.option_icon = self.g_conf.get_string("/apps/battery_status/icon_policy")
		if self.option_percentage_low is None:
			self.g_conf.set_int("/apps/battery_status/percentage_low", 8)
			self.option_percentage_low = self.g_conf.get_int("/apps/battery_status/percentage_low")
		if self.option_timer is None:
			self.g_conf.set_int("/apps/battery_status/timer", 60)
			self.option_timer = self.g_conf.get_int("/apps/battery_status/timer")
		# get g-p-m icon policy setting for advice to remove tray icon, if in use.
		self.option_gpm_icon_policy = self.g_conf.get_string("/apps/gnome-power-manager/ui/icon_policy")
		# add GConf settings of applet for detecting changes
		self.g_conf.add_dir("/apps/battery_status", gconf.CLIENT_PRELOAD_NONE)
		# set up related callbacks for changes of GConf settings
		self.g_conf.notify_add("/apps/battery_status/icon_policy", self.init_gconf_changes, "icon")
		self.g_conf.notify_add("/apps/battery_status/lock_screen", self.init_gconf_changes, "lock")
		self.g_conf.notify_add("/apps/battery_status/show_info", self.init_gconf_changes, "info")
		self.g_conf.notify_add("/apps/battery_status/show_sleep", self.init_gconf_changes, "sleep")
		self.g_conf.notify_add("/apps/battery_status/show_session", self.init_gconf_changes, "session")
		self.g_conf.notify_add("/apps/battery_status/show_power", self.init_gconf_changes, "power")
		self.g_conf.notify_add("/apps/battery_status/percentage_low", self.init_gconf_changes, "low")
		self.g_conf.notify_add("/apps/battery_status/timer", self.init_gconf_changes, "timer")
		self.g_conf.notify_add("/apps/battery_status/first_run", self.init_gconf_changes, "run")
		self.g_conf.notify_add("/apps/battery_status/show_powermode", self.init_gconf_changes, "cpufreq")
		self.g_conf.notify_add("/apps/battery_status/power_source_sensitive", self.init_gconf_changes, "sensitive")
		self.g_conf.notify_add("/apps/battery_status/power_manager", self.init_gconf_changes, "powermanager")
		self.g_conf.notify_add("/apps/gnome-power-manager/ui/icon_policy", self.init_gconf_changes, "gpm_icon")
	
	
	def init_gconf_changes(self, client, connection_id, entry, option):
		"""Callback function for GConf changes"""
		### update GConf settings on the fly, if it's changing within indicator works
		# update icon policy option
		if option == "icon" and entry.get_value().type == gconf.VALUE_STRING:
			if entry.get_value().get_string() == "low" or entry.get_value().get_string() == "never" or entry.get_value().get_string() == "charge":
				self.option_icon = entry.get_value().get_string()
			else:
				self.option_icon = "always"
			if self.option_icon == "never" and self.option_show == "icon":
				# prevent to hide icon
				self.g_conf.set_string("/apps/battery_status/icon_policy", "always")
		# update lock screen option
		elif option == "lock" and entry.get_value().type == gconf.VALUE_BOOL:
			self.option_lock = entry.get_value().get_bool()
		# update info option
		elif option == "info" and entry.get_value().type == gconf.VALUE_BOOL:
			self.option_info = entry.get_value().get_bool()
		# update option for show sleep actions in main menu
		elif option == "sleep" and entry.get_value().type == gconf.VALUE_BOOL:
			self.option_sleep = entry.get_value().get_bool()
			self.update_main_menu(option)
		# update option for show session actions in main menu
		elif option == "session" and entry.get_value().type == gconf.VALUE_BOOL:
			self.option_session = entry.get_value().get_bool()
			self.update_main_menu(option)
		# update option for show session actions in main menu
		elif option == "power" and entry.get_value().type == gconf.VALUE_BOOL:
			self.option_power = entry.get_value().get_bool()
			self.update_main_menu(option)
		# update low percentage option
		elif option == "low" and entry.get_value().type == gconf.VALUE_INT:
			self.option_percentage_low = entry.get_value().get_int()
		# update timer option
		elif option == "timer" and entry.get_value().type == gconf.VALUE_INT:
			self.option_timer = entry.get_value().get_int()
		# update first run option
		elif option == "run" and entry.get_value().type == gconf.VALUE_BOOL:
			self.option_firstrun = entry.get_value().get_bool()
		# update show powermode option
		elif option == "cpufreq" and entry.get_value().type == gconf.VALUE_BOOL:
			self.option_cpufreq = entry.get_value().get_bool()
			self.update_main_menu(option)
		# update gpm option
		elif option == "powermanager" and entry.get_value().type == gconf.VALUE_BOOL:
			self.option_gpm = entry.get_value().get_bool()
		# update power source sensitive option
		elif option == "sensitive" and entry.get_value().type == gconf.VALUE_BOOL:
			self.option_sensitive = entry.get_value().get_bool()
		if not self.power_error:
			try:
				# to handle error, if it's occurs before detecting self.power_error state
				self.power_dbus_signal()
			except:
				pass
		else:
			self.power_dbus_signal_error("dbus")
	
	
	def update_main_menu(self, item):
		"""Update main menu items and its state"""
		# update Power Modes and its separator according to current preferences
		if (item == "sleep" or item == "session" or item == "cpufreq"):
			if self.option_sleep or self.option_session or not self.option_cpufreq or not self.package_cpufreq or self.cpufreq_error or not self.option_power:
				self.power_separator.hide()
			else:
				self.power_separator.show()
		else:
			pass
		# update sleep items
		if item == "sleep":
			if self.option_sleep and self.can_suspend and self.can_hibernate and not self.power_error:
				self.item_sleep_separator.show()
				self.item_suspend.show()
				self.item_hibernate.show()
			else:
				self.item_sleep_separator.hide()
				self.item_suspend.hide()
				self.item_hibernate.hide()
		# update session items
		elif item == "session":
			if self.option_session and self.package_session:
				self.item_session_separator.show()
				self.item_logout.show()
				self.item_shutdown.show()
			else:
				self.item_session_separator.hide()
				self.item_logout.hide()
				self.item_shutdown.hide()
		# update power items
		elif item == "power":
			if self.option_power and self.package_power:
				self.item_power.set_sensitive(True)
				self.item_pmsettings.show()
				self.power_separator.show()
			else:
				self.item_power.set_sensitive(False)
				self.item_pmsettings.hide()
				self.power_separator.hide()
		# update CPU frequency scaling items
		elif item == "cpufreq":
			if self.option_cpufreq and self.package_cpufreq and not self.cpufreq_error:
				self.item_mode_powersave.show()
				self.item_mode_ondemand.show()
				self.item_mode_conservative.show()
				self.item_mode_performance.show()
				# detect current CPU frequency scaling mode
				if self.powermode == "powersave":
					self.item_mode_powersave.set_property("active", True)
				elif self.powermode == "ondemand":
					self.item_mode_ondemand.set_property("active", True)
				elif self.powermode == "conservative":
					self.item_mode_conservative.set_property("active", True)
				elif self.powermode == "performance":
					self.item_mode_performance.set_property("active", True)
			else:
				self.item_mode_powersave.hide()
				self.item_mode_ondemand.hide()
				self.item_mode_conservative.hide()
				self.item_mode_performance.hide()
		else:
			pass
	
	
	def init_main_menu(self):
		"""Create main menu"""
		### init main menu with items
		menu = gtk.Menu()
		# menu item: ----
		item_separator = gtk.SeparatorMenuItem()
		item_separator.show()
		# menu item: Power Management...
		self.item_pmsettings = gtk.MenuItem("_Power Management...", True)
		self.item_pmsettings.connect("activate", self.main_menu_action)
		self.item_pmsettings.show()
		# menu item: Battery Status
		self.item_status = gtk.MenuItem()
		self.item_status.set_label("Battery Status")
		self.item_status.set_sensitive(True)
		self.item_status.connect("activate", self.main_menu_battery_dialog)
		self.item_status.show()
		# menu item: Power Source
		self.item_power = gtk.MenuItem()
		self.item_power.set_label("Power Source:")
		self.item_power.connect("activate", self.main_menu_action)
		self.item_power.show()
		# Powersave item for Power Mode
		self.item_mode_powersave.connect("toggled", self.powermode_powersave)
		# Ondemand item for Power Mode
		self.item_mode_ondemand.connect("toggled", self.powermode_ondemand)
		# Conservative item for Power Mode
		self.item_mode_conservative.connect("toggled", self.powermode_conservative)
		# Performance item for Power Mode
		self.item_mode_performance.connect("toggled", self.powermode_performance)
		### add basic items in menu
		menu.append(self.item_status)
		menu.append(item_separator)
		menu.append(self.item_power)
		menu.append(self.item_mode_powersave)
		menu.append(self.item_mode_ondemand)
		menu.append(self.item_mode_conservative)
		menu.append(self.item_mode_performance)
		menu.append(self.power_separator)
		menu.append(self.item_pmsettings)
		### show sleep actions in menu
		# menu item: ----
		self.item_sleep_separator = gtk.SeparatorMenuItem()
		self.item_sleep_separator.show()
		menu.append(self.item_sleep_separator)
		# menu item: Suspend
		self.item_suspend = gtk.MenuItem("S_uspend", True)
		self.item_suspend.connect("activate", self.suspend)
		self.item_suspend.show()
		# menu item: Hibernate
		self.item_hibernate = gtk.MenuItem("Hiber_nate", True)
		self.item_hibernate.connect("activate", self.hibernate)
		self.item_hibernate.show()
		### add sleep items in menu
		menu.append(self.item_suspend)
		menu.append(self.item_hibernate)
		### show session actions in menu
		# menu item: ----
		self.item_session_separator = gtk.SeparatorMenuItem()
		self.item_session_separator.show()
		# menu item: Log Out...
		self.item_logout = gtk.MenuItem("_Log Out...", True)
		self.item_logout.connect("activate", self.main_menu_action)
		self.item_logout.show()
		# menu item: Shut Down...
		self.item_shutdown = gtk.MenuItem("_Shut Down...", True)
		self.item_shutdown.connect("activate", self.main_menu_action)
		self.item_shutdown.show()
		# add session items in menu
		menu.append(self.item_session_separator)
		menu.append(self.item_logout)
		menu.append(self.item_shutdown)
		### set main menu
		self.indicator.set_menu(menu)
		### update status of menu items
		self.update_main_menu("sleep")
		self.update_main_menu("session")
		self.update_main_menu("power")
		self.update_main_menu("cpufreq")
	
	
	def main_menu_battery_dialog(self, item = False):
		"""Battery device information dialog"""
		# local callback for expander of battery technical info frame
		def expander_info_cb(expander, gparam_bool, widget):
			if expander.get_expanded():
				widget.show()
			else:
				widget.hide()
			# update GConf setting
			self.g_conf.set_bool("/apps/battery_status/show_info", expander.get_expanded())
		# local callback for destroying battery dialog
		def dialog_destroy(arg):
			# unlock menu
			self.lock_battery_status = False
			# make battery item active, only if battery available
			if self.no_battery or self.power_error:
				self.item_status.set_sensitive(False)
			else:
				self.item_status.set_sensitive(True)
		# lock main menu for prevent duplicating battery information dialog
		self.lock_battery_status = True
		# update power information
		self.power_dbus_signal()
		# text column for capacity data
		text_capacity = "State:\nPercentage:\nCapacity:\nLifetime:\nCharge time:\n"
		# text column for charge data
		text_charge = "Voltage:\nRate:\nCurrent:\nLast full:\nMaximum:\n"
		# text column for battery info
		text_info = "Technology:\nVendor:\nModel:\nSerial:"
		# label for capacity text
		label_capacity = gtk.Label()
		label_capacity.set_markup(text_capacity)
		label_capacity.set_justify(gtk.JUSTIFY_LEFT)
		label_capacity.show()
		# show label with capacity data
		self.label_capacity_data.show()
		# hbox for capacity labels
		hbox_capacity = gtk.HBox()
		hbox_capacity.add(label_capacity)
		hbox_capacity.add(self.label_capacity_data)
		hbox_capacity.show()
		# show progress bar of current battery capacity
		self.progress_capacity.show()
		# vbox for hbox with capacity labels and capacity progress bar
		vbox_capacity = gtk.VBox()
		vbox_capacity.add(hbox_capacity)
		vbox_capacity.add(self.progress_capacity)
		vbox_capacity.show()
		# alignment for vbox with capacity data
		align_capacity = gtk.Alignment()
		align_capacity.set_property("left-padding", 12)
		align_capacity.set_property("right-padding", 12)
		align_capacity.set_property("top-padding", 12)
		align_capacity.set_property("bottom-padding", 12)
		align_capacity.add(vbox_capacity)
		align_capacity.show()
		# frame for alignment
		frame_capacity = gtk.Frame("  Battery Capacity  ")
		frame_capacity.set_border_width(8)
		frame_capacity.add(align_capacity)
		frame_capacity.show()
		# label for charge text
		label_charge = gtk.Label()
		label_charge.set_markup(text_charge)
		label_charge.set_justify(gtk.JUSTIFY_LEFT)
		label_charge.show()
		# show label with charge data
		self.label_charge_data.show()
		# hbox for charge labels
		hbox_charge = gtk.HBox()
		hbox_charge.add(label_charge)
		hbox_charge.add(self.label_charge_data)
		hbox_charge.show()
		# show progress bar of current battery charge
		self.progress_charge.show()
		# vbox for hbox with charge labels and charge progress bar
		vbox_charge = gtk.VBox()
		vbox_charge.add(hbox_charge)
		vbox_charge.add(self.progress_charge)
		vbox_charge.show()
		# alignment for vbox with charge data
		align_charge = gtk.Alignment()
		align_charge.set_property("left-padding", 12)
		align_charge.set_property("right-padding", 12)
		align_charge.set_property("top-padding", 12)
		align_charge.set_property("bottom-padding", 12)
		align_charge.add(vbox_charge)
		align_charge.show()
		# frame for alignment
		frame_charge = gtk.Frame("  Battery Charge  ")
		frame_charge.set_border_width(8)
		frame_charge.add(align_charge)
		frame_charge.show()
		# show label with battery info text
		self.label_info.show()
		# show label with battery info data
		self.label_info_data.show()
		# hbox for battery info labels
		hbox_info = gtk.HBox()
		hbox_info.add(self.label_info)
		hbox_info.add(self.label_info_data)
		hbox_info.show()
		# alignment for hbox with battery info
		align_info = gtk.Alignment()
		align_info.set_property("left-padding", 24)
		align_info.set_property("right-padding", 0)
		align_info.set_property("top-padding", 12)
		align_info.set_property("bottom-padding", 12)
		align_info.add(hbox_info)
		align_info.show()
		# frame for alignment
		frame_info = gtk.Frame()
		frame_info.set_border_width(8)
		frame_info.add(align_info)
		# expander for frame with battery info
		expander_info = gtk.Expander()
		if self.option_info:
			frame_info.show()
		else:
			frame_info.hide()
		expander_info.set_expanded(self.option_info)
		expander_info.set_label("Battery Information")
		expander_info.connect("notify::expanded", expander_info_cb, frame_info)
		expander_info.show()
		# generate dialog window with provided battery information
		message_battery_status = gtk.Dialog(title = "", parent = None, buttons = None)
		message_battery_status.set_title("Battery Status")
		message_battery_status.set_property("gravity", gtk.gdk.GRAVITY_NORTH)
		message_battery_status.set_property("skip-taskbar-hint", False)
		message_battery_status.set_property("skip-pager-hint", True)
		message_battery_status.set_property("resizable", False)
		message_battery_status.set_property("deletable", False)
		message_battery_status.set_has_separator(False)
		message_battery_status.set_icon(self.pixbuf_battery_info)
		message_battery_status.vbox.add(frame_capacity)
		message_battery_status.vbox.add(frame_charge)
		message_battery_status.vbox.add(expander_info)
		message_battery_status.vbox.add(frame_info)
		# copy dialog for destroy it, if user remove applet when dialog open
		self.lock_battery_status = message_battery_status
		# connect battery dialog to destroy event
		message_battery_status.connect("destroy", dialog_destroy)
		# show and present battery dialog
		message_battery_status.show()
		message_battery_status.present()
	
	
	def main_menu_action(self, item):
		"""Main menu items/actions handler"""
		action = item.get_label()
		### Power Source item
		if action.find("Po_wer Source:") != -1:
			if self.option_sensitive:
				# if power_source_sensitive, then set statistics for current power source device
				if action.find("Battery") != -1:
					self.g_conf.set_string("/apps/gnome-power-manager/info/last_device", self.power_address + "/devices/" + self.power_battery_name)
				else:
					self.g_conf.set_string("/apps/gnome-power-manager/info/last_device", self.power_address + "/devices/" + self.power_ac_name)
			# show power statistics
			os.system("gnome-power-statistics &")
		### Power Management item
		elif action == "_Power Management...":
			# show power preferences
			os.system("gnome-power-preferences &")
		### Session items
		elif action == "_Log Out...":
			# show logout dialog
			os.system("gnome-session-save --logout-dialog &")
		elif action == "_Shut Down...":
			# show shutdown dialog
			os.system("gnome-session-save --shutdown-dialog &")
		else:
			pass
	
	
	def init_power_dbus(self):
		"""Init D-Bus UPower/DeviceKit-Power environment"""
		# set up default values for global D-Bus UPower/DeviceKit-Power related variables
		self.power_ac_address = None
		self.power_battery_address = None
		self.power_ac_name = "line_power"
		self.power_battery_name = "battery"
		self.properties_bus = "org.freedesktop.DBus.Properties"
		# set up power info dictionaries
		self.power_device_types = {0: "Unknown", 1: "Line Power", 2: "Battery", 3: "UPS", 4: "Monitor", 5: "Mouse", 6: "Keyboard", 7: "PDA", 8: "Phone"}
		self.power_battery_techs = {0: "Unknown", 1: "Lithium Ion", 2: "Lithium Polymer", 3: "Lithium Iron Phosphate", 4: "Lead Acid", 5: "Nickel Cadmium", 6: "Nickel Metal Hydride"}
		self.power_battery_state = {0: "unknown", 1: "charging", 2: "discharging", 3: "discharged", 4: "charged"}
		self.power_battery_tooltip = {0: "", 1: "until charged", 2: "remaining", 3: "", 4: "is fully charged"}
		self.power_source = {0: "Po_wer Source: AC Line", 1: "Po_wer Source: Battery"}
		self.battery_full = 4
		try:
			# to create connection for system bus
			self.bus = dbus.SystemBus()
			try:
				# to get power object using new UPower D-Bus address
				self.power_address = "/org/freedesktop/UPower"
				self.power_bus = "org.freedesktop.UPower"
				power_object = self.bus.get_object(self.power_bus, self.power_address)
			except:
				# try to get power object using old DeviceKit-Power D-Bus address
				self.power_address = "/org/freedesktop/DeviceKit/Power"
				self.power_bus = "org.freedesktop.DeviceKit.Power"
				power_object = self.bus.get_object(self.power_bus, self.power_address)
			# setting up power interfaces
			self.power_interface = dbus.Interface(power_object, self.power_bus)
			self.power_properties_interface = dbus.Interface(power_object, self.properties_bus)
			# update power information
			self.power_dbus_signal()
			# connect to signals
			self.power_interface.connect_to_signal("DeviceChanged", self.power_dbus_signal)
			self.power_interface.connect_to_signal("DeviceAdded", self.power_dbus_signal)
			self.power_interface.connect_to_signal("DeviceRemoved", self.power_dbus_signal)
			# if everything good
			self.power_error = False
		except dbus.exceptions.DBusException, exception:
			# else show error message
			self.dbus_error(exception.__str__())
		if self.power_error:
			self.power_dbus_signal_error("dbus")
	
	
	def dbus_error(self, exception_text):
		"""Critical message with technical details, when something goes wrong"""
		# text buffer with details
		text_buffer = gtk.TextBuffer()
		text_buffer.set_text("DBus exception:\n" + exception_text)
		# container for text buffer
		text_field = gtk.TextView()
		text_field.set_editable(False)
		text_field.set_wrap_mode(gtk.WRAP_WORD)
		text_field.set_buffer(text_buffer)
		text_field.show()
		# container for details
		message_details = gtk.Expander()
		message_details.set_expanded(False)
		message_details.set_label("View technical details")
		message_details.add(text_field)
		message_details.show()
		# generate error message dialog
		message_error = gtk.MessageDialog(parent = None, type = gtk.MESSAGE_WARNING, buttons = gtk.BUTTONS_NONE, message_format = None)
		title_text = "Battery Indicator error."
		message_text = "Indicator can't get power state of computer. D-Bus system or UPower/DeviceKit-Power information unavailable now.\n"
		question_text = "Would you like still to keep indicator?"
		message_error.set_markup('<span weight="bold" size="medium">' + title_text + '</span>' + '\n\n' + message_text + '\n' + question_text)
		message_error.set_icon(self.pixbuf_gpm_32)
		message_error.set_property("window-position", "center")
		message_error.add_button("_Remove", 1)
		message_error.add_button("_Keep", 0)
		message_error.set_default_response(0)
		message_error.set_keep_above(True)
		message_error.vbox.add(message_details)
		# show error dialog
		if message_error.run():
			# quit with error status
			sys.exit(1)
		message_error.destroy()
		self.power_error = True
	
	
	def power_dbus_signal_error(self, error_type):
		"""Handle power errors"""
		### Keep applet working, even if some power information not available
		if type(self.lock_battery_status).__name__ == "Dialog":
			# destroy battery status dialog, if active
			self.lock_battery_status.destroy()
		# set up icon
		self.indicator.set_icon("gpm-ac-adapter")
		# show text in main menu items and tooltip according to current error
		if error_type == "dbus":
			self.item_status.set_label("No Information")
			self.item_power.set_label("Power Source: No Information")
		elif error_type == "no_battery":
			self.item_status.set_label("No Battery")
			self.item_power.set_label("Power Source: AC Line")
		# make battery information dialog unavailable
		self.item_status.set_sensitive(False)
	
	
	def power_dbus_signal(self, device = None):
		"""Get battery power information on update"""
		# get list of available power devices
		power_devices = self.power_interface.EnumerateDevices()
		# list for AC devices
		power_ac_devices = []
		# list for battery devices
		power_battery_devices = []
		# detect ACs and batteries
		for power_device in power_devices:
			if power_device.find(self.power_battery_name) != -1:
				power_battery_devices.append(power_device)
			if power_device.find(self.power_ac_name) != -1:
				power_ac_devices.append(power_device)
		# detect primary AC address from available AC devices
		if len(power_ac_devices) == 0:
			self.power_ac_address = ""
		elif len(power_ac_devices) == 1:
			# get info for available AC
			power_ac_object = self.bus.get_object(self.power_bus, power_ac_devices[0])
			power_ac_interface = dbus.Interface(power_ac_object, self.properties_bus)
			power_ac = power_ac_interface.GetAll("")
			# check type of available AC
			if self.power_device_types[power_ac['Type']] == self.power_device_types[1] and power_ac['PowerSupply']:
				self.power_ac_address = power_ac_devices[0]
			else:
				self.power_ac_address = ""
		else:
			# multiple AC power lines - is it possible?
			pass
		# detect battery address from available battery devices
		if len(power_battery_devices) == 0:
			self.power_battery_address = ""
		elif len(power_battery_devices) == 1:
			# get info for available battery
			power_battery_object = self.bus.get_object(self.power_bus, power_battery_devices[0])
			self.power_battery_interface = dbus.Interface(power_battery_object, self.properties_bus)
			power_battery = self.power_battery_interface.GetAll("")
			# check type of available battery
			if self.power_device_types[power_battery['Type']] == self.power_device_types[2] and power_battery['PowerSupply']:
				self.power_battery_address = power_battery_devices[0]
			else:
				self.power_battery_address = ""
		else:
			# FIXME: code for processing multiple batteries goes here
			# it hasn't been tested on real multiple batteries, so any bug reports and logs are welcome
			for battery_address in power_battery_devices:
				power_battery_object = self.bus.get_object(self.power_bus, battery_address)
				self.power_battery_interface = dbus.Interface(power_battery_object, self.properties_bus)
				power_battery = self.power_battery_interface.GetAll("")
				# finding laptop battery address
				if self.power_device_types[power_battery['Type']] == self.power_device_types[2] and power_battery['PowerSupply']:
					self.power_battery_address = battery_address
					break
		# check availability of battery
		if self.power_battery_address:
			# get battery information, if available
			power_battery_object = self.bus.get_object(self.power_bus, self.power_battery_address)
			self.power_battery_interface = dbus.Interface(power_battery_object, self.properties_bus)
			self.no_battery = False
		else:
			# keep working without battery information
			self.no_battery = True
			self.power_dbus_signal_error("no_battery")
			return False
		### keep working with battery information
		if self.lock_battery_status or self.power_error or self.no_battery:
			self.item_status.set_sensitive(False)
		else:
			self.item_status.set_sensitive(True)
		# get sleep actions state
		self.can_suspend = self.power_properties_interface.Get(self.power_address, "can-suspend")
		self.can_hibernate = self.power_properties_interface.Get(self.power_address, "can-hibernate")
		# update sleep menu items according its state
		if self.option_sleep and (self.can_suspend or self.can_hibernate):
			self.item_sleep_separator.show()
			if self.can_suspend:
				self.item_suspend.show()
			else:
				self.item_suspend.hide()
			if self.can_hibernate:
				self.item_hibernate.show()
			else:
				self.item_hibernate.hide()
		else:
			self.item_sleep_separator.hide()
			self.item_suspend.hide()
			self.item_hibernate.hide()
		# update battery information
		power_battery = self.power_battery_interface.GetAll("")
		# update power properties
		power_properties = self.power_properties_interface.GetAll("")
		# get battery state
		self.battery_state = self.power_battery_state[power_battery['State']]
		# get battery percentage
		self.battery_percent = int(round(power_battery['Percentage'], 1))
		# detect power source
		self.power_on_battery = power_properties['OnBattery']
		# detect critical charge
		if not self.on_low_battery and self.power_on_battery and self.battery_percent < self.option_percentage_low and self.option_timer > 0:
			self.on_low_battery = True
			self.critical_low()
		elif self.battery_percent >= self.option_percentage_low:
			self.on_low_battery = False
		# generate icon name according to current percentage
		if self.battery_percent >= 0 and self.battery_percent < 10:
			self.icon_name = "gpm-battery-000"
		elif self.battery_percent >= 10 and self.battery_percent < 20:
			self.icon_name = "gpm-battery-020"
		elif self.battery_percent >= 20 and self.battery_percent < 48:
			self.icon_name = "gpm-battery-040"
		elif self.battery_percent >= 48 and self.battery_percent < 75:
			self.icon_name = "gpm-battery-060"
		elif self.battery_percent >= 75 and self.battery_percent < 89:
			self.icon_name = "gpm-battery-080"
		elif self.battery_percent >= 89 and self.battery_percent <= 100:
			self.icon_name = "gpm-battery-100"
		if self.battery_state == self.power_battery_state[self.battery_full]:
			self.icon_name = "gpm-battery-charged"
		elif self.battery_state == "charging":
			self.icon_name += "-charging"
		# set up battery icon for main icon widget
		self.indicator.set_icon(self.icon_name)
		if self.option_icon == "always" or (self.option_icon == "low" and self.on_low_battery) or (self.option_icon == "charge" and self.battery_state != self.power_battery_state[self.battery_full]):
			self.indicator.set_status(appindicator.STATUS_ACTIVE)
		else:
			self.indicator.set_status(appindicator.STATUS_PASSIVE)
		# set up battery icon for battery status dialog
		self.pixbuf_battery_info = self.icon_theme.load_icon(self.icon_name, 128, gtk.ICON_LOOKUP_FORCE_SVG)
		# init local variable for tooltip
		battery_time = "-:--"
		# get battery lifetime
		self.battery_lifetime = power_battery['TimeToEmpty']
		# convert battery lifetime from seconds to human-readable format (if available)
		if self.battery_lifetime:
			self.battery_lifetime = time.strftime('%H:%M', time.gmtime(self.battery_lifetime))[1:5]
			battery_time = self.battery_lifetime
		else:
			self.battery_lifetime = "-:--"
			h_time = 0
			m_time = 0
		# get battery charge time
		self.battery_chargetime = power_battery['TimeToFull']
		# convert battery charge time from seconds to human-readable format (if available)
		if self.battery_chargetime:
			self.battery_chargetime = time.strftime('%H:%M', time.gmtime(self.battery_chargetime))[1:5]
			battery_time = self.battery_chargetime
		else:
			self.battery_chargetime = "-:--"
		# convert battery percentage from integer to string
		self.battery_percent = str(self.battery_percent)
		# generate text for battery status item in main menu
		self.status_text = ""
		if battery_time != "-:--" and self.battery_state != self.power_battery_state[self.battery_full]:
			# show battery lifetime/chargetime only if it available and this actual
			self.status_text += battery_time
			if self.power_on_battery:
				self.status_text += " remaining"
			else:
				self.status_text += " until full"
		else:
			# in other case, show battery status
			self.status_text += "Battery is " + self.power_battery_state[power_battery['State']]
		self.status_text += ": " + self.battery_percent + '%'
		# update battery status text in main menu
		self.item_status.set_label(self.status_text)
		# generate text for power source item in main menu
		self.power_source_text = self.power_source[self.power_on_battery]
		# update power source text in main menu
		self.item_power.set_label(self.power_source_text)
		### get information for battery status dialog, if it's open
		if self.lock_battery_status:
			# generate text about battery capacity
			text_capacity_data = self.power_battery_state[power_battery['State']] + '\n'
			text_capacity_data += str(round(power_battery['Percentage'], 1)) + '%\n'
			text_capacity_data += str(round(power_battery['Capacity'], 1)) + '%\n'
			text_capacity_data += self.battery_lifetime + '\n'
			text_capacity_data += self.battery_chargetime + '\n'
			# generate text about battery charge
			text_charge_data = str(round(power_battery['Voltage'], 1)) + ' V   \n'
			text_charge_data += str(round(power_battery['EnergyRate'], 1)) + ' W  \n'
			text_charge_data += str(round(power_battery['Energy'], 1)) + ' Wh\n'
			text_charge_data += str(round(power_battery['EnergyFull'], 1)) + ' Wh\n'
			text_charge_data += str(round(power_battery['EnergyFullDesign'], 1)) + ' Wh\n'
			# update battery status in information dialog window
			self.label_capacity_data.set_markup(text_capacity_data)
			self.label_charge_data.set_markup(text_charge_data)
			# update progress bar with battery capacity information
			self.progress_capacity.set_fraction(power_battery['Percentage']/100)
			self.progress_capacity.set_text(str(int(round(power_battery['Percentage'], 1))) + '%')
			# update progress bar with battery charge information
			progress_charge_data = 100 * power_battery['Energy'] / power_battery['EnergyFullDesign']
			self.progress_charge.set_fraction(progress_charge_data/100)
			self.progress_charge.set_text(str(int(round(progress_charge_data, 1))) + '%')
			if self.option_info:
				# text column for battery info
				text_info = "Technology:\nVendor:\nModel:"
				# generate text about battery information
				text_info_data = '\t' + self.power_battery_techs[power_battery['Technology']] + '\n'
				text_info_data += '\t' + power_battery['Vendor'] + '\n'
				text_info_data += '\t' + power_battery['Model']
				# display serial number only if available
				if power_battery['Serial'] != "":
					text_info += "\nSerial:"
					text_info_data += '\n' + '\t' + power_battery['Serial']
				self.label_info.set_markup(text_info)
				self.label_info_data.set_markup(text_info_data)
		# update power management state and its current settings
		self.power_management()
	
	
	def critical_low(self):
		"""Message dialog about low battery percentage"""
		# local function for timer management
		def count(widget, timeout):
			# update power information
			self.power_dbus_signal()
			# if timer enabled, then handle it
			if timeout:
				self.timer -= 1
				self.opacity -= self.opacity_step
				if self.timer > 1:
					second = " seconds."
				else:
					second = " second."
				if self.timer > 0 and self.power_on_battery:
					# show critical low message only if timer ticking and AC not available
					message_warning_label.set_markup("This message will automatically disappear in " + str(self.timer) + second)
					widget.set_property("opacity", self.opacity)
					return True
				else:
					# in other case close message and stop timer
					message_battery_low.destroy()
					return False
			# if timer not enabled
			else:
				# but AC become available
				if not self.power_on_battery:
					# close message
					message_battery_low.destroy()
					return False
				# in other case keep message showing and updating power information
				else:
					return True
		# setting up default timer
		self.timer = self.option_timer
		# setting up opacity variables for fade out effect
		self.opacity = 1
		self.opacity_step = float(self.opacity)/self.timer
		# create icon for message
		battery_low_pixbuf = self.icon_theme.load_icon("gpm-battery-000", 64, gtk.ICON_LOOKUP_FORCE_SVG)
		battery_low_icon = gtk.Image()
		battery_low_icon.set_from_pixbuf(battery_low_pixbuf)
		battery_low_icon.show()
		# generate message about low battery percentage
		message_battery_low = gtk.MessageDialog(parent = None, buttons = gtk.BUTTONS_NONE, message_format = None)
		title_text = "You are now running on reserve battery power."
		message_text = "Please connect your computer to AC power. If you do not,\nyour computer will go to sleep in a few minutes to preserve\nthe contents of memory."
		message_battery_low.set_markup('<span weight="bold" size="medium">' + title_text + '</span>' + '\n\n' + message_text)
		message_battery_low.set_icon(self.pixbuf_gpm_32)
		message_battery_low.set_property("gravity", gtk.gdk.GRAVITY_NORTH)
		message_battery_low.set_keep_above(True)
		message_battery_low.set_image(battery_low_icon)
		# add Hibernate Now button, if such feature available
		if self.can_hibernate:
			message_battery_low.add_button("Hiber_nate Now", 2)
		# add Ignore button and set up it by default
		message_battery_low.add_button("Ign_ore", 1)
		message_battery_low.set_default_response(1)
		# add timer and label for destroy dialog
		if self.timer > 1:
			message_battery_low.set_property("opacity", self.opacity)
			message_warning_label = gtk.Label()
			message_warning_label.set_markup("This message will automatically disappear in " + str(self.timer) + " seconds.")
			message_warning_label.show()
			message_battery_low.vbox.add(message_warning_label)
			gobject.timeout_add_seconds(self.timer, message_battery_low.destroy)
			timeout = True
		elif self.timer == 1:
			message_battery_low.set_property("opacity", 0.8)
			timeout = False
		gobject.timeout_add_seconds(1, count, message_battery_low, timeout)
		if message_battery_low.run() == 2:
			self.hibernate()
		message_battery_low.destroy()
	
	
	def hibernate(self, data = None):
		"""Hibernate call for UPower/DeviceKit-Power D-Bus"""
		try:
			if self.package_screensaver and self.option_lock:
				# to lock screen and
				os.system("gnome-screensaver-command --lock &")
			# to hibernate
			self.power_interface.Hibernate()
		except dbus.exceptions.DBusException, exception:
			# if hibernate fail, then do nothing
			pass
	
	
	def suspend(self, data = None):
		"""Suspend call for UPower/DeviceKit-Power D-Bus"""
		try:
			if self.package_screensaver and self.option_lock:
				# to lock screen and
				os.system("gnome-screensaver-command --lock &")
			# to suspend
			self.power_interface.Suspend()
		except dbus.exceptions.DBusException, exception:
			# if suspend fail, then do nothing
			pass
	
	
	def powermode_powersave(self, item):
		"""Power settings management for Powersave CPU frequency scaling mode"""
		if self.powermode != "powersave":
			for cpu in self.cpu_items:
				os.system("cpufreq-selector -g powersave -c " + cpu.rstrip('\n'))
			cpu_mode = "powersave"
			cpu_switch = True
		else:
			cpu_mode = ""
			cpu_switch = False
		# update cpu frequency information
		self.init_cpufreq()
		# if power state has changed, then switch settings, if available
		if cpu_switch and cpu_mode == self.powermode and self.package_power and self.option_gpm:
			self.power_settings(self.power_dev, cpu_mode)
	
	
	def powermode_ondemand(self, item):
		"""Power settings management for Ondemand CPU frequency scaling mode"""
		if self.powermode != "ondemand":
			for cpu in self.cpu_items:
				os.system("cpufreq-selector -g ondemand -c " + cpu.rstrip('\n'))
			cpu_mode = "ondemand"
			cpu_switch = True
		else:
			cpu_mode = ""
			cpu_switch = False
		# update cpu frequency information
		self.init_cpufreq()
		# if power state has changed, then switch settings, if available
		if cpu_switch and cpu_mode == self.powermode and self.package_power and self.option_gpm:
			self.power_settings(self.power_dev, cpu_mode)
	
	
	def powermode_conservative(self, item):
		"""Power settings management for Conservative CPU frequency scaling mode"""
		if self.powermode != "conservative":
			for cpu in self.cpu_items:
				os.system("cpufreq-selector -g conservative -c " + cpu.rstrip('\n'))
			cpu_mode = "conservative"
			cpu_switch = True
		else:
			cpu_mode = ""
			cpu_switch = False
		# update cpu frequency information
		self.init_cpufreq()
		# if power state has changed, then switch settings, if available
		if cpu_switch and cpu_mode == self.powermode and self.package_power and self.option_gpm:
			self.power_settings(self.power_dev, cpu_mode)
	
	
	def powermode_performance(self, item):
		"""Power settings management for Performance CPU frequency scaling mode"""
		if self.powermode != "performance":
			for cpu in self.cpu_items:
				os.system("cpufreq-selector -g performance -c " + cpu.rstrip('\n'))
			cpu_mode = "performance"
			cpu_switch = True
		else:
			cpu_mode = ""
			cpu_switch = False
		# update cpu frequency information
		self.init_cpufreq()
		# if power state has changed, then switch settings, if available
		if cpu_switch and cpu_mode == self.powermode and self.package_power and self.option_gpm:
			self.power_settings(self.power_dev, cpu_mode)
	
	
	def power_management(self):
		"""Power settings management for changing power source"""
		cpu_mode = self.powermode
		power_switch = True
		if self.power_on_battery and self.power_dev != "battery":
			self.power_dev = "battery"
		elif not self.power_on_battery and self.power_dev != "ac":
			self.power_dev = "ac"
		else:
			power_switch = False
		# update cpu frequency information
		self.init_cpufreq()
		# if power state has changed, then switch settings, if available
		if power_switch and cpu_mode == self.powermode and self.package_power and self.option_gpm:
			self.power_settings(self.power_dev, cpu_mode)
	
	
	def power_settings(self, power_source, cpu_mode):
		"""Switch power settings according to current power source/cpu frequency"""
		if power_source == "ac":
			if cpu_mode == "powersave":
				self.g_conf.set_bool("/apps/gnome-power-manager/backlight/idle_dim_ac", False)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/brightness_ac", 70)
				self.g_conf.set_bool("/apps/gnome-power-manager/disk/spindown_enable_ac", False)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_computer_ac", 1800)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_display_ac", 300)
			elif cpu_mode == "ondemand":
				self.g_conf.set_bool("/apps/gnome-power-manager/backlight/idle_dim_ac", False)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/brightness_ac", 80)
				self.g_conf.set_bool("/apps/gnome-power-manager/disk/spindown_enable_ac", False)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_computer_ac", 3600)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_display_ac", 600)
			elif cpu_mode == "conservative":
				self.g_conf.set_bool("/apps/gnome-power-manager/backlight/idle_dim_ac", False)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/brightness_ac", 90)
				self.g_conf.set_bool("/apps/gnome-power-manager/disk/spindown_enable_ac", False)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_computer_ac", 7200)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_display_ac", 1800)
			elif cpu_mode == "performance":
				self.g_conf.set_bool("/apps/gnome-power-manager/backlight/idle_dim_ac", False)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/brightness_ac", 100)
				self.g_conf.set_bool("/apps/gnome-power-manager/disk/spindown_enable_ac", False)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_computer_ac", 0)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_display_ac", 3600)
			else:
				pass
		elif power_source == "battery":
			if cpu_mode == "powersave":
				self.g_conf.set_bool("/apps/gnome-power-manager/backlight/battery_reduce", True)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/brightness_dim_battery", 60)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/idle_brightness", 20)
				self.g_conf.set_bool("/apps/gnome-power-manager/backlight/idle_dim_battery", True)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/idle_dim_time", 10)
				self.g_conf.set_bool("/apps/gnome-power-manager/disk/spindown_enable_battery", True)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_computer_battery", 600)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_display_battery", 60)
			elif cpu_mode == "ondemand":
				self.g_conf.set_bool("/apps/gnome-power-manager/backlight/battery_reduce", True)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/brightness_dim_battery", 40)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/idle_brightness", 40)
				self.g_conf.set_bool("/apps/gnome-power-manager/backlight/idle_dim_battery", True)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/idle_dim_time", 30)
				self.g_conf.set_bool("/apps/gnome-power-manager/disk/spindown_enable_battery", True)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_computer_battery", 1800)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_display_battery", 300)
			elif cpu_mode == "conservative":
				self.g_conf.set_bool("/apps/gnome-power-manager/backlight/battery_reduce", True)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/brightness_dim_battery", 20)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/idle_brightness", 60)
				self.g_conf.set_bool("/apps/gnome-power-manager/backlight/idle_dim_battery", True)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/idle_dim_time", 60)
				self.g_conf.set_bool("/apps/gnome-power-manager/disk/spindown_enable_battery", False)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_computer_battery", 3600)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_display_battery", 600)
			elif cpu_mode == "performance":
				self.g_conf.set_bool("/apps/gnome-power-manager/backlight/battery_reduce", False)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/brightness_dim_battery", 0)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/idle_brightness", 80)
				self.g_conf.set_bool("/apps/gnome-power-manager/backlight/idle_dim_battery", True)
				self.g_conf.set_int("/apps/gnome-power-manager/backlight/idle_dim_time", 90)
				self.g_conf.set_bool("/apps/gnome-power-manager/disk/spindown_enable_battery", False)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_computer_battery", 7200)
				self.g_conf.set_int("/apps/gnome-power-manager/timeout/sleep_display_battery", 1800)
			else:
				pass
		else:
			pass


